@@ -43,6 +43,7 @@
 
 #include "oggz_private.h"
 #include "oggz_byteorder.h"
+#include "dirac.h"
 
 #include <oggz/oggz_stream.h>
 
@@ -62,8 +63,6 @@ int oggz_set_metric_linear (OGGZ * oggz, long serialno,
 #define INT32_BE_AT(x) _be_32((*(ogg_int32_t *)(x)))
 #define INT64_LE_AT(x) _le_64((*(ogg_int64_t *)(x)))
 
-#define OGGZ_AUTO_MULT 1000Ull
-
 static int
 oggz_stream_set_numheaders (OGGZ * oggz, long serialno, int numheaders)
 {
@@ -80,13 +79,13 @@ oggz_stream_set_numheaders (OGGZ * oggz, long serialno, int numheaders)
 }
 
 static int
-auto_speex (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
+auto_speex (OGGZ * oggz, long serialno, unsigned char * data, long length, void * user_data)
 {
-  unsigned char * header = op->packet;
+  unsigned char * header = data;
   ogg_int64_t granule_rate = 0;
   int numheaders;
 
-  if (op->bytes < 68) return 0;
+  if (length < 68) return 0;
 
   granule_rate = (ogg_int64_t) INT32_LE_AT(&header[36]);
 #ifdef DEBUG
@@ -95,6 +94,8 @@ auto_speex (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
 
   oggz_set_granulerate (oggz, serialno, granule_rate, OGGZ_AUTO_MULT);
 
+  oggz_set_preroll (oggz, serialno, 3);
+
   numheaders = (ogg_int64_t) INT32_LE_AT(&header[68]) + 2;
   oggz_stream_set_numheaders (oggz, serialno, numheaders);
 
@@ -102,12 +103,12 @@ auto_speex (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
 }
 
 static int
-auto_vorbis (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
+auto_vorbis (OGGZ * oggz, long serialno, unsigned char * data, long length, void * user_data)
 {
-  unsigned char * header = op->packet;
+  unsigned char * header = data;
   ogg_int64_t granule_rate = 0;
 
-  if (op->bytes < 30) return 0;
+  if (length < 30) return 0;
 
   granule_rate = (ogg_int64_t) INT32_LE_AT(&header[12]);
 #ifdef DEBUG
@@ -116,6 +117,8 @@ auto_vorbis (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
 
   oggz_set_granulerate (oggz, serialno, granule_rate, OGGZ_AUTO_MULT);
 
+  oggz_set_preroll (oggz, serialno, 2);
+
   oggz_stream_set_numheaders (oggz, serialno, 3);
 
   return 1;
@@ -133,15 +136,15 @@ static int intlog(int num) {
 #endif
 
 static int
-auto_theora (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
+auto_theora (OGGZ * oggz, long serialno, unsigned char * data, long length, void * user_data)
 {
-  unsigned char * header = op->packet;
+  unsigned char * header = data;
   ogg_int32_t fps_numerator, fps_denominator;
   char keyframe_granule_shift = 0;
   int keyframe_shift;
 
   /* TODO: this should check against 42 for the relevant version numbers */
-  if (op->bytes < 41) return 0;
+  if (length < 41) return 0;
 
   fps_numerator = INT32_BE_AT(&header[22]);
   fps_denominator = INT32_BE_AT(&header[26]);
@@ -171,13 +174,14 @@ auto_theora (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
 			OGGZ_AUTO_MULT * (ogg_int64_t)fps_denominator);
   oggz_set_granuleshift (oggz, serialno, keyframe_shift);
 
+
   oggz_stream_set_numheaders (oggz, serialno, 3);
 
   return 1;
 }
 
 static int
-auto_annodex (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
+auto_annodex (OGGZ * oggz, long serialno, unsigned char * data, long length, void * user_data)
 {
   /* Apply a zero metric */
   oggz_set_granulerate (oggz, serialno, 0, 1);
@@ -186,12 +190,12 @@ auto_annodex (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
 }
 
 static int
-auto_anxdata (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
+auto_anxdata (OGGZ * oggz, long serialno, unsigned char * data, long length, void * user_data)
 {
-  unsigned char * header = op->packet;
+  unsigned char * header = data;
   ogg_int64_t granule_rate_numerator = 0, granule_rate_denominator = 0;
 
-  if (op->bytes < 28) return 0;
+  if (length < 28) return 0;
 
   granule_rate_numerator = INT64_LE_AT(&header[8]);
   granule_rate_denominator = INT64_LE_AT(&header[16]);
@@ -208,17 +212,17 @@ auto_anxdata (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
 }
 
 static int
-auto_flac0 (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
+auto_flac0 (OGGZ * oggz, long serialno, unsigned char * data, long length, void * user_data)
 {
-  unsigned char * header = op->packet;
+  unsigned char * header = data;
   ogg_int64_t granule_rate = 0;
 
-  granule_rate = (ogg_int64_t) (header[14] << 12) | (header[15] << 4) | 
+  granule_rate = (ogg_int64_t) (header[14] << 12) | (header[15] << 4) |
             ((header[16] >> 4)&0xf);
 #ifdef DEBUG
     printf ("Got flac rate %d\n", (int)granule_rate);
 #endif
-    
+
   oggz_set_granulerate (oggz, serialno, granule_rate, OGGZ_AUTO_MULT);
 
   oggz_stream_set_numheaders (oggz, serialno, 3);
@@ -227,15 +231,15 @@ auto_flac0 (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
 }
 
 static int
-auto_flac (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
+auto_flac (OGGZ * oggz, long serialno, unsigned char * data, long length, void * user_data)
 {
-  unsigned char * header = op->packet;
+  unsigned char * header = data;
   ogg_int64_t granule_rate = 0;
   int numheaders;
 
-  if (op->bytes < 51) return 0;
+  if (length < 51) return 0;
 
-  granule_rate = (ogg_int64_t) (header[27] << 12) | (header[28] << 4) | 
+  granule_rate = (ogg_int64_t) (header[27] << 12) | (header[28] << 4) |
             ((header[29] >> 4)&0xf);
 #ifdef DEBUG
   printf ("Got flac rate %d\n", (int)granule_rate);
@@ -254,12 +258,12 @@ auto_flac (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
  * http://wiki.xiph.org/index.php/OggPCM2
  */
 static int
-auto_oggpcm2 (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
+auto_oggpcm2 (OGGZ * oggz, long serialno, unsigned char * data, long length, void * user_data)
 {
-  unsigned char * header = op->packet;
+  unsigned char * header = data;
   ogg_int64_t granule_rate;
 
-  if (op->bytes < 28) return 0;
+  if (length < 28) return 0;
 
   granule_rate = (ogg_int64_t) INT32_BE_AT(&header[16]);
 #ifdef DEBUG
@@ -274,13 +278,13 @@ auto_oggpcm2 (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
 }
 
 static int
-auto_celt (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
+auto_celt (OGGZ * oggz, long serialno, unsigned char * data, long length, void * user_data)
 {
-  unsigned char * header = op->packet;
+  unsigned char * header = data;
   ogg_int64_t granule_rate = 0;
   int numheaders;
 
-  if (op->bytes < 56) return 0;
+  if (length < 56) return 0;
 
   granule_rate = (ogg_int64_t) INT32_LE_AT(&header[40]);
 #ifdef DEBUG
@@ -296,17 +300,17 @@ auto_celt (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
 }
 
 static int
-auto_cmml (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
+auto_cmml (OGGZ * oggz, long serialno, unsigned char * data, long length, void * user_data)
 {
-  unsigned char * header = op->packet;
+  unsigned char * header = data;
   ogg_int64_t granule_rate_numerator = 0, granule_rate_denominator = 0;
   int granuleshift;
 
-  if (op->bytes < 28) return 0;
+  if (length < 28) return 0;
 
   granule_rate_numerator = INT64_LE_AT(&header[12]);
   granule_rate_denominator = INT64_LE_AT(&header[20]);
-  if (op->bytes > 28)
+  if (length > 28)
     granuleshift = (int)header[28];
   else
     granuleshift = 0;
@@ -327,14 +331,14 @@ auto_cmml (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
 }
 
 static int
-auto_kate (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
+auto_kate (OGGZ * oggz, long serialno, unsigned char * data, long length, void * user_data)
 {
-  unsigned char * header = op->packet;
+  unsigned char * header = data;
   ogg_int32_t gps_numerator, gps_denominator;
   unsigned char granule_shift = 0;
   int numheaders;
 
-  if (op->bytes < 64) return 0;
+  if (length < 64) return 0;
 
   gps_numerator = INT32_LE_AT(&header[24]);
   gps_denominator = INT32_LE_AT(&header[28]);
@@ -350,21 +354,44 @@ auto_kate (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
   oggz_set_granulerate (oggz, serialno, gps_numerator,
 			OGGZ_AUTO_MULT * gps_denominator);
   oggz_set_granuleshift (oggz, serialno, granule_shift);
- 
+
   oggz_stream_set_numheaders (oggz, serialno, numheaders);
 
   return 1;
 }
 
 static int
-auto_fisbone (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
+auto_dirac (OGGZ * oggz, long serialno, unsigned char * data, long length, void * user_data)
 {
-  unsigned char * header = op->packet;
+  int granule_shift = 22; /* not a typo */
+  dirac_info *info;
+
+  info = oggz_malloc(sizeof(dirac_info));
+  if (info == NULL) return -1;
+
+  dirac_parse_info(info, data, length);
+
+  /* the granulerate is twice the frame rate (in order to handle interlace) */
+  oggz_set_granulerate (oggz, serialno,
+	2 * (ogg_int64_t)info->fps_numerator,
+	OGGZ_AUTO_MULT * (ogg_int64_t)info->fps_denominator);
+  oggz_set_granuleshift (oggz, serialno, granule_shift);
+
+  oggz_stream_set_numheaders (oggz, serialno, 0);
+
+  oggz_free(info);
+  return 1;
+}
+
+static int
+auto_fisbone (OGGZ * oggz, long serialno, unsigned char * data, long length, void * user_data)
+{
+  unsigned char * header = data;
   long fisbone_serialno; /* The serialno referred to in this fisbone */
   ogg_int64_t granule_rate_numerator = 0, granule_rate_denominator = 0;
   int granuleshift, numheaders;
 
-  if (op->bytes < 48) return 0;
+  if (length < 48) return 0;
 
   fisbone_serialno = (long) INT32_LE_AT(&header[12]);
 
@@ -376,7 +403,7 @@ auto_fisbone (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
   granuleshift = (int)header[48];
 
 #ifdef DEBUG
-  printf ("Got fisbone granulerate %lld/%lld, granuleshift %d for serialno %010ld\n",
+  printf ("Got fisbone granulerate %lld/%lld, granuleshift %d for serialno %010lu\n",
 	  granule_rate_numerator, granule_rate_denominator, granuleshift,
 	  fisbone_serialno);
 #endif
@@ -389,23 +416,18 @@ auto_fisbone (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
   /* Increment the number of headers for this stream */
   numheaders = oggz_stream_get_numheaders (oggz, serialno);
   oggz_stream_set_numheaders (oggz, serialno, numheaders+1);
-				
+
   return 1;
 }
 
 static int
-auto_fishead (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
+auto_fishead (OGGZ * oggz, long serialno, unsigned char * data, long length, void * user_data)
 {
-  if (!op->b_o_s)
-  {
-    return auto_fisbone(oggz, op, serialno, user_data);
-  }
-  
   oggz_set_granulerate (oggz, serialno, 0, 1);
 
   /* For skeleton, numheaders will get incremented as each header is seen */
   oggz_stream_set_numheaders (oggz, serialno, 1);
-  
+
   return 1;
 }
 
@@ -419,27 +441,28 @@ typedef struct {
   int encountered_first_data_packet;
 } auto_calc_speex_info_t;
 
-static ogg_int64_t 
+static ogg_int64_t
 auto_calc_speex(ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
-  
+
   /*
    * on the first (b_o_s) packet, set calculate_data to be the number
    * of speex frames per packet
    */
 
-  auto_calc_speex_info_t *info 
+  auto_calc_speex_info_t *info
           = (auto_calc_speex_info_t *)stream->calculate_data;
 
   if (stream->calculate_data == NULL) {
-    stream->calculate_data = malloc(sizeof(auto_calc_speex_info_t));
+    stream->calculate_data = oggz_malloc(sizeof(auto_calc_speex_info_t));
+    if (stream->calculate_data == NULL) return -1;
     info = stream->calculate_data;
     info->encountered_first_data_packet = 0;
-    info->packet_size = 
+    info->packet_size =
             (*(int *)(op->packet + 64)) * (*(int *)(op->packet + 56));
     info->headers_encountered = 1;
     return 0;
   }
-  
+
   if (info->headers_encountered < 2) {
     info->headers_encountered += 1;
   } else {
@@ -454,7 +477,7 @@ auto_calc_speex(ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
     if (stream->last_granulepos > 0) {
       return stream->last_granulepos + info->packet_size;
     }
-    
+
     return -1;
   }
 
@@ -472,19 +495,21 @@ typedef struct {
   int encountered_first_data_packet;
 } auto_calc_celt_info_t;
 
-static ogg_int64_t 
+static ogg_int64_t
 auto_calc_celt (ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
-  
+
   /*
    * on the first (b_o_s) packet, set calculate_data to be the number
    * of celt frames per packet
    */
 
-  auto_calc_celt_info_t *info 
+  auto_calc_celt_info_t *info
           = (auto_calc_celt_info_t *)stream->calculate_data;
 
   if (stream->calculate_data == NULL) {
-    stream->calculate_data = malloc(sizeof(auto_calc_celt_info_t));
+    stream->calculate_data = oggz_malloc(sizeof(auto_calc_celt_info_t));
+    if (stream->calculate_data == NULL) return -1;
+
     info = stream->calculate_data;
     info->encountered_first_data_packet = 0;
 
@@ -497,7 +522,7 @@ auto_calc_celt (ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
     info->headers_encountered = 1;
     return 0;
   }
-  
+
   if (info->headers_encountered < 2) {
     info->headers_encountered += 1;
   } else {
@@ -512,7 +537,7 @@ auto_calc_celt (ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
     if (stream->last_granulepos > 0) {
       return stream->last_granulepos + info->packet_size;
     }
-    
+
     return -1;
   }
 
@@ -523,9 +548,8 @@ auto_calc_celt (ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
  * Header packets are marked by a set MSB in the first byte.  Inter packets
  * are marked by a set 2MSB in the first byte.  Intra packets (keyframes)
  * are any that are left over ;-)
- * 
- * (see http://www.theora.org/doc/Theora_I_spec.pdf for the theora 
- * specification)
+ *
+ * (see http://theora.org/doc/Theora.pdf for the theora specification)
  */
 
 typedef struct {
@@ -533,7 +557,7 @@ typedef struct {
 } auto_calc_theora_info_t;
 
 
-static ogg_int64_t 
+static ogg_int64_t
 auto_calc_theora(ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
 
   long keyframe_no;
@@ -549,13 +573,14 @@ auto_calc_theora(ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
   if (first_byte & 0x80)
   {
     if (info == NULL) {
-      stream->calculate_data = malloc(sizeof(auto_calc_theora_info_t));
+      stream->calculate_data = oggz_malloc(sizeof(auto_calc_theora_info_t));
+      if (stream->calculate_data == NULL) return -1;
       info = stream->calculate_data;
     }
     info->encountered_first_data_packet = 0;
     return (ogg_int64_t)0;
   }
-  
+
   /* known granulepos */
   if (now > (ogg_int64_t)(-1)) {
     info->encountered_first_data_packet = 1;
@@ -582,7 +607,7 @@ auto_calc_theora(ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
     return stream->last_granulepos + 1;
   }
 
-  keyframe_shift = stream->granuleshift; 
+  keyframe_shift = stream->granuleshift;
   /*
    * retrieve last keyframe number
    */
@@ -592,12 +617,12 @@ auto_calc_theora(ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
    */
   keyframe_no += (stream->last_granulepos & ((1 << keyframe_shift) - 1)) + 1;
   return ((ogg_int64_t)keyframe_no) << keyframe_shift;
-  
+
 
 }
 
 static ogg_int64_t
-auto_rcalc_theora(ogg_int64_t next_packet_gp, oggz_stream_t *stream, 
+auto_rcalc_theora(ogg_int64_t next_packet_gp, oggz_stream_t *stream,
                   ogg_packet *this_packet, ogg_packet *next_packet) {
 
   int keyframe = (int)(next_packet_gp >> stream->granuleshift);
@@ -638,9 +663,9 @@ auto_rcalc_theora(ogg_int64_t next_packet_gp, oggz_stream_t *stream,
  * (additional information is not required)
  *
  * The two blocksizes can be determined from the first header packet, by reading
- * byte 28.  1 << (packet[28] >> 4) == long_size.  
+ * byte 28.  1 << (packet[28] >> 4) == long_size.
  * 1 << (packet[28] & 0xF) == short_size.
- * 
+ *
  * (see http://xiph.org/vorbis/doc/Vorbis_I_spec.html for specification)
  */
 
@@ -654,13 +679,14 @@ typedef struct {
   int log2_num_modes;
   int mode_sizes[1];
 } auto_calc_vorbis_info_t;
-        
 
-static ogg_int64_t 
+
+static ogg_int64_t
 auto_calc_vorbis(ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
 
   auto_calc_vorbis_info_t *info;
-  
+  int ii;
+
   if (stream->calculate_data == NULL) {
     /*
      * on the first (b_o_s) packet, determine the long and short sizes,
@@ -668,11 +694,13 @@ auto_calc_vorbis(ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
      */
     int short_size;
     int long_size;
-  
+
     long_size = 1 << (op->packet[28] >> 4);
     short_size = 1 << (op->packet[28] & 0xF);
 
-    stream->calculate_data = malloc(sizeof(auto_calc_vorbis_info_t));
+    stream->calculate_data = oggz_malloc(sizeof(auto_calc_vorbis_info_t));
+    if (stream->calculate_data == NULL) return -1;
+
     info = (auto_calc_vorbis_info_t *)stream->calculate_data;
     info->nln_increments[3] = long_size >> 1;
     info->nln_increments[2] = 3 * (long_size >> 2) - (short_size >> 2);
@@ -694,10 +722,10 @@ auto_calc_vorbis(ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
     /*
      * the code pages, a whole bunch of other fairly useless stuff, AND,
      * RIGHT AT THE END (of a bunch of variable-length compressed rubbish that
-     * basically has only one actual set of values that everyone uses BUT YOU 
+     * basically has only one actual set of values that everyone uses BUT YOU
      * CAN'T BE SURE OF THAT, OH NO YOU CAN'T) is the only piece of data that's
      * actually useful to us - the packet modes (because it's inconceivable to
-     * think people might want _just that_ and nothing else, you know, for 
+     * think people might want _just that_ and nothing else, you know, for
      * seeking and stuff).
      *
      * Fortunately, because of the mandate that non-used bits must be zero
@@ -712,11 +740,12 @@ auto_calc_vorbis(ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
       int size_check;
       int *mode_size_ptr;
       int i;
-      
-      /* 
+      size_t size_realloc_bytes;
+
+      /*
        * This is the format of the mode data at the end of the packet for all
        * Vorbis Version 1 :
-       * 
+       *
        * [ 6:number_of_modes ]
        * [ 1:size | 16:window_type(0) | 16:transform_type(0) | 8:mapping ]
        * [ 1:size | 16:window_type(0) | 16:transform_type(0) | 8:mapping ]
@@ -729,7 +758,7 @@ auto_calc_vorbis(ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
        * 0 0 0 0 0 1 0 0
        * 0 0 1 0 0 0 0 0
        * 0 0 1 0 0 0 0 0
-       * 0 0 1|0 0 0 0 0 
+       * 0 0 1|0 0 0 0 0
        * 0 0 0 0|0|0 0 0
        * 0 0 0 0 0 0 0 0
        * 0 0 0 0|0 0 0 0
@@ -739,11 +768,11 @@ auto_calc_vorbis(ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
        * 0 0 0 0 0 0 0 0 V
        * 0 0 0|0 0 0 0 0
        * 0 0 0 0 0 0 0 0
-       * 0 0 1|0 0 0 0 0 
-       * 0 0|1|0 0 0 0 0 
-       * 
-       *  
-       * i.e. each entry is an important bit, 32 bits of 0, 8 bits of blah, a 
+       * 0 0 1|0 0 0 0 0
+       * 0 0|1|0 0 0 0 0
+       *
+       *
+       * i.e. each entry is an important bit, 32 bits of 0, 8 bits of blah, a
        * bit of 1.
        * Let's find our last 1 bit first.
        *
@@ -761,7 +790,7 @@ auto_calc_vorbis(ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
 
       while (1)
       {
-        
+
         /*
          * from current_pos-5:(offset+1) to current_pos-1:(offset+1) should
          * be zero
@@ -769,15 +798,15 @@ auto_calc_vorbis(ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
         offset = (offset + 7) % 8;
         if (offset == 7)
           current_pos -= 1;
-        
-        if 
+
+        if
         (
           ((current_pos[-5] & ~((1 << (offset + 1)) - 1)) != 0)
           ||
-          current_pos[-4] != 0 
-          || 
-          current_pos[-3] != 0 
-          || 
+          current_pos[-4] != 0
+          ||
+          current_pos[-3] != 0
+          ||
           current_pos[-2] != 0
           ||
           ((current_pos[-1] & ((1 << (offset + 1)) - 1)) != 0)
@@ -785,38 +814,56 @@ auto_calc_vorbis(ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
         {
           break;
         }
-        
-        size += 1;
-        
-        current_pos -= 5;
-        
-      } 
 
-      if (offset > 4) {
-        size_check = (current_pos[0] >> (offset - 5)) & 0x3F;
-      } else {
-        /* mask part of byte from current_pos */
-        size_check = (current_pos[0] & ((1 << (offset + 1)) - 1));
-        /* shift to appropriate position */
-        size_check <<= (5 - offset);
-        /* or in part of byte from current_pos - 1 */
-        size_check |= (current_pos[-1] & ~((1 << (offset + 3)) - 1)) >> 
-                (offset + 3);
+        size += 1;
+
+        current_pos -= 5;
+
       }
-      
-      size_check += 1;
+
+      /* Give ourselves a chance to recover if we went back too far by using
+       * the size check. */
+      for (ii=0; ii < 2; ii++) {
+       if (offset > 4) {
+         size_check = (current_pos[0] >> (offset - 5)) & 0x3F;
+       } else {
+         /* mask part of byte from current_pos */
+         size_check = (current_pos[0] & ((1 << (offset + 1)) - 1));
+         /* shift to appropriate position */
+         size_check <<= (5 - offset);
+         /* or in part of byte from current_pos - 1 */
+         size_check |= (current_pos[-1] & ~((1 << (offset + 3)) - 1)) >>
+           (offset + 3);
+       }
+
+       size_check += 1;
+       if (size_check == size) {
+         break;
+       }
+        offset = (offset + 1) % 8;
+        if (offset == 0)
+          current_pos += 1;
+       current_pos += 5;
+       size -= 1;
+      }
+
+#ifdef DEBUG
       if (size_check != size)
       {
         printf("WARNING: size parsing failed for VORBIS mode packets\n");
       }
+#endif
+
+      /* Check that size to be realloc'd doesn't overflow */
+      size_realloc_bytes = sizeof(auto_calc_vorbis_info_t) + (size - 1) * sizeof(int);
+      if (size_realloc_bytes < sizeof (auto_calc_vorbis_info_t)) return -1;
+
+      /* Store mode size information in our info struct */
+      info = realloc(stream->calculate_data, size_realloc_bytes);
+      if (info == NULL) return -1;
+
+      stream->calculate_data = info;
 
-      /*
-       * store mode size information in our info struct
-       */
-      stream->calculate_data = realloc(stream->calculate_data,
-              sizeof(auto_calc_vorbis_info_t) + (size - 1) * sizeof(int));
-      info = (auto_calc_vorbis_info_t *)(stream->calculate_data);
-      
       i = -1;
       while ((1 << (++i)) < size);
       info->log2_num_modes = i;
@@ -831,9 +878,9 @@ auto_calc_vorbis(ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
         *mode_size_ptr++ = (current_pos[0] >> offset) & 0x1;
         current_pos += 5;
       }
-      
+
     }
-    
+
     return 0;
   }
 
@@ -841,7 +888,7 @@ auto_calc_vorbis(ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
 
   return -1;
 
-  { 
+  {
     /*
      * we're in a data packet!  First we need to get the mode of the packet,
      * and from the mode, the size
@@ -852,7 +899,7 @@ auto_calc_vorbis(ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
 
     mode = (op->packet[0] >> 1) & ((1 << info->log2_num_modes) - 1);
     size = info->mode_sizes[mode];
-  
+
     /*
      * if we have a working granulepos, we use it, but only if we can't
      * calculate a valid gp value.
@@ -878,28 +925,28 @@ auto_calc_vorbis(ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op) {
       return -1;
     }
 
-    result = stream->last_granulepos + 
+    result = stream->last_granulepos +
       (
-        (info->last_was_long ? info->long_size  : info->short_size) 
-        + 
+        (info->last_was_long ? info->long_size  : info->short_size)
+        +
         (size ? info->long_size : info->short_size)
       ) / 4;
     info->last_was_long = size;
 
     return result;
-    
+
   }
-  
+
 }
 
 ogg_int64_t
 auto_rcalc_vorbis(ogg_int64_t next_packet_gp, oggz_stream_t *stream,
                   ogg_packet *this_packet, ogg_packet *next_packet) {
 
-  auto_calc_vorbis_info_t *info = 
+  auto_calc_vorbis_info_t *info =
                   (auto_calc_vorbis_info_t *)stream->calculate_data;
 
-  int mode = 
+  int mode =
       (this_packet->packet[0] >> 1) & ((1 << info->log2_num_modes) - 1);
   int this_size = info->mode_sizes[mode] ? info->long_size : info->short_size;
   int next_size;
@@ -934,13 +981,15 @@ typedef struct {
   int encountered_first_data_packet;
 } auto_calc_flac_info_t;
 
-static ogg_int64_t 
+static ogg_int64_t
 auto_calc_flac (ogg_int64_t now, oggz_stream_t *stream, ogg_packet *op)
 {
   auto_calc_flac_info_t *info;
 
   if (stream->calculate_data == NULL) {
-    stream->calculate_data = malloc(sizeof(auto_calc_flac_info_t));
+    stream->calculate_data = oggz_malloc(sizeof(auto_calc_flac_info_t));
+    if (stream->calculate_data == NULL) return -1;
+
     info = (auto_calc_flac_info_t *)stream->calculate_data;
     info->previous_gp = 0;
     info->encountered_first_data_packet = 0;
@@ -1039,31 +1088,32 @@ const oggz_auto_contenttype_t oggz_auto_codec_ident[] = {
   {"Annodex", 8, "Annodex", auto_annodex, NULL, NULL},
   {"fishead", 7, "Skeleton", auto_fishead, NULL, NULL},
   {"fLaC", 4, "Flac0", auto_flac0, auto_calc_flac, NULL},
-  {"\177FLAC", 4, "Flac", auto_flac, auto_calc_flac, NULL},
+  {"\177FLAC", 5, "Flac", auto_flac, auto_calc_flac, NULL},
   {"AnxData", 7, "AnxData", auto_anxdata, NULL, NULL},
   {"CELT    ", 8, "CELT", auto_celt, auto_calc_celt, NULL},
   {"\200kate\0\0\0", 8, "Kate", auto_kate, NULL, NULL},
+  {"BBCD\0", 5, "Dirac", auto_dirac, NULL, NULL},
   {"", 0, "Unknown", NULL, NULL, NULL}
-}; 
+};
 
 static int
 oggz_auto_identify (OGGZ * oggz, long serialno, unsigned char * data, long len)
 {
   int i;
-  
+
   for (i = 0; i < OGGZ_CONTENT_UNKNOWN; i++)
   {
     const oggz_auto_contenttype_t *codec = oggz_auto_codec_ident + i;
-    
+
     if (len >= codec->bos_str_len &&
         memcmp (data, codec->bos_str, codec->bos_str_len) == 0) {
-      
+
       oggz_stream_set_content (oggz, serialno, i);
-      
+
       return 1;
     }
   }
-                      
+
   oggz_stream_set_content (oggz, serialno, OGGZ_CONTENT_UNKNOWN);
   return 0;
 }
@@ -1081,27 +1131,44 @@ oggz_auto_identify_packet (OGGZ * oggz, ogg_packet * op, long serialno)
 }
 
 int
-oggz_auto_get_granulerate (OGGZ * oggz, ogg_packet * op, long serialno, 
-                void * user_data)
+oggz_auto_read_bos_page (OGGZ * oggz, ogg_page * og, long serialno,
+                         void * user_data)
 {
   int content = 0;
 
   content = oggz_stream_get_content(oggz, serialno);
   if (content < 0 || content >= OGGZ_CONTENT_UNKNOWN) {
     return 0;
+  } else if (content == OGGZ_CONTENT_SKELETON && !ogg_page_bos(og)) {
+    return auto_fisbone(oggz, serialno, og->body, og->body_len, user_data);
+  } else {
+    return oggz_auto_codec_ident[content].reader(oggz, serialno, og->body, og->body_len, user_data);
   }
-
-  oggz_auto_codec_ident[content].reader(oggz, op, serialno, user_data);
-  return 0;
 }
 
-ogg_int64_t 
-oggz_auto_calculate_granulepos(int content, ogg_int64_t now, 
+int
+oggz_auto_read_bos_packet (OGGZ * oggz, ogg_packet * op, long serialno,
+                           void * user_data)
+{
+  int content = 0;
+
+  content = oggz_stream_get_content(oggz, serialno);
+  if (content < 0 || content >= OGGZ_CONTENT_UNKNOWN) {
+    return 0;
+  } else if (content == OGGZ_CONTENT_SKELETON && !op->b_o_s) {
+    return auto_fisbone(oggz, serialno, op->packet, op->bytes, user_data);
+  } else {
+    return oggz_auto_codec_ident[content].reader(oggz, serialno, op->packet, op->bytes, user_data);
+  }
+}
+
+ogg_int64_t
+oggz_auto_calculate_granulepos(int content, ogg_int64_t now,
                 oggz_stream_t *stream, ogg_packet *op) {
   if (oggz_auto_codec_ident[content].calculator != NULL) {
     ogg_int64_t r = oggz_auto_codec_ident[content].calculator(now, stream, op);
     return r;
-  } 
+  }
 
   return now;
 }
