@@ -1,12 +1,17 @@
 #include "config.h"
 
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h> /* For off_t not found in stdio.h */
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 #include "httpdate.h"
 
 #define CONTENT_TYPE_OGG "Content-Type: application/ogg\n"
+#define ACCEPT_TIMEURI_OGG "X-Accept-TimeURI: application/ogg\n"
 
 int
 header_last_modified (time_t mtime)
@@ -25,11 +30,17 @@ header_not_modified (void)
 }
 
 int
-header_content_type_ogg ()
+header_content_type_ogg (void)
 {
   return printf (CONTENT_TYPE_OGG);
 }
 
+int
+header_accept_timeuri_ogg (void)
+{
+  return printf (ACCEPT_TIMEURI_OGG);
+}
+
 int
 header_content_length (off_t len)
 {
