@@ -75,7 +75,8 @@ track_state_new (void)
 {
   OCTrackState * ts;
 
-  ts = (OCTrackState *) malloc (sizeof(*ts));
+  if ((ts = (OCTrackState *) malloc (sizeof(*ts))) == NULL)
+    return NULL;
 
   memset (ts, 0, sizeof(*ts));
 
@@ -103,7 +104,8 @@ track_state_add (OggzTable * state, long serialno)
 {
   OCTrackState * ts;
 
-  ts = track_state_new ();
+  if ((ts = track_state_new ()) == NULL)
+    return NULL;
 
   if (oggz_table_insert (state, serialno, ts) == ts) {
     return ts;
@@ -146,11 +148,21 @@ _ogg_page_copy (const ogg_page * og)
 {
   ogg_page * new_og;
 
-  new_og = malloc (sizeof (*og));
-  new_og->header = malloc (og->header_len);
+  if ((new_og = malloc (sizeof (*og))) == NULL)
+    return NULL;
+
+  if ((new_og->header = malloc (og->header_len)) == NULL) {
+    free (new_og);
+    return NULL;
+  }
   new_og->header_len = og->header_len;
   memcpy (new_og->header, og->header, og->header_len);
-  new_og->body = malloc (og->body_len);
+
+  if ((new_og->body = malloc (og->body_len)) == NULL) {
+    free (new_og->header);
+    free (new_og);
+    return NULL;
+  }
   new_og->body_len = og->body_len;
   memcpy (new_og->body, og->body, og->body_len);
 
@@ -168,21 +180,26 @@ _ogg_page_free (const ogg_page * og)
 }
 
 static void
-_ogg_page_set_eos (ogg_page * og)
+_ogg_page_set_eos (const ogg_page * og)
 {
   if (og == NULL) return;
 
   og->header[5] |= 0x04;
-  ogg_page_checksum_set (og);
+  ogg_page_checksum_set ((ogg_page *)og);
 }
 
 static void
-fwrite_ogg_page (FILE * outfile, const ogg_page * og)
+fwrite_ogg_page (OCState * state, const ogg_page * og)
 {
+  size_t n;
+
   if (og == NULL) return;
 
-  fwrite (og->header, 1, og->header_len, outfile);
-  fwrite (og->body, 1, og->body_len, outfile);
+  if (!state->dry_run) {
+    n = fwrite (og->header, 1, og->header_len, state->outfile);
+    if (n == (size_t)og->header_len)
+      n = fwrite (og->body, 1, og->body_len, state->outfile);
+  }
 }
 
 /************************************************************
@@ -199,8 +216,14 @@ page_accum_new (const ogg_page * og, double time)
 {
   OCPageAccum * pa;
 
-  pa = malloc(sizeof (*pa));
-  pa->og = _ogg_page_copy (og);
+  if ((pa = malloc(sizeof (*pa))) == NULL)
+    return NULL;
+
+  if ((pa->og = _ogg_page_copy (og)) == NULL) {
+    free (pa);
+    return NULL;
+  }
+
   pa->time = time;
 
   return pa;
@@ -242,10 +265,17 @@ track_state_advance_page_accum (OCTrackState * ts)
   int i, accum_size;
   int e, earliest_new; /* Index into page accumulator of the earliest page that
                         * contains a packet from the new GOP */
+  int succ_continued; /* Successor page is continued */
   OCPageAccum * pa;
 
   if (ts == NULL || ts->page_accum == NULL) return 0;
 
+  /* Upon entry, the next page is the one most recently read; we only get here
+   * if that page is continued, otherwise the accumulator is simply cleared,
+   * in read_gs() and read_dirac() below.
+   */
+  succ_continued = 1;
+
   earliest_new = accum_size = oggz_table_size (ts->page_accum);
 
   /* Working backwards through the page accumulator ... */
@@ -253,19 +283,38 @@ track_state_advance_page_accum (OCTrackState * ts)
     pa = (OCPageAccum *) oggz_table_lookup (ts->page_accum, i);
 
     /* If we have a page with granulepos, it necessarily contains the end
-     * of a packet from an earlier GOP, and thus this is the last page that
-     * we need to recover.
+     * of a packet from an earlier GOP, and may contain the start of a packet
+     * from the new GOP. If so, it is the earliest page to recover.
      */
     if (ogg_page_granulepos (pa->og) != -1) {
       earliest_new = i;
+
+      /* If the successor page was not continued, ie. it began a packet,
+       * then this page only contains packets from the previous GOP or earlier.
+       * In this case, we can be sure that the successor is the earliest_new
+       * page to use.
+       */
+      if (!succ_continued)
+        earliest_new++;
+
+      /* We are working backwards, so we can break out when we have found 
+       * the earliest page to recover.
+       */
       break;
     }
+
+    /* Update succ_continued flag; we are working backwards, so this page
+     * is the successor to the one we will consider on the next iteration.
+     */
+    succ_continued = ogg_page_continued(OGG_PAGE_CONST(pa->og));
   }
 
   /* If all accumulated pages have no granulepos, keep them,
    * and do not modify start_granule */
   if (earliest_new == 0) return accum_size;
 
+  /* WTF: This check is from my original implementation of this function in
+   * changeset:3557. How can it arise? -Conrad 20081028 */
   if (earliest_new > accum_size)
     earliest_new = accum_size;
 
@@ -311,6 +360,8 @@ skeleton_write_packet (OCState * state, ogg_packet * op)
   return ret;
 }
 
+/* MIME types, for use in Skeleton. This must be in the same order as
+ * enum OggzStreamContent defined in <oggz/oggz_constants.h> */
 const char * type_names[] = {
   "video/theora",
   "audio/vorbis",
@@ -324,6 +375,7 @@ const char * type_names[] = {
   NULL, /* AnxData */
   "audio/celt",
   "application/x-kate",
+  "video/dirac",
   NULL /* UNKNOWN */
 };
 
@@ -352,7 +404,9 @@ fisbone_init (OGGZ * oggz, OCState * state, OCTrackState * ts, long serialno)
     content_type = oggz_stream_get_content (oggz, serialno);
     name = type_names[content_type];
     len = snprintf (NULL, 0, CONTENT_TYPE_FMT, name);
-    ts->fisbone.message_header_fields = malloc(len+1);
+    if ((ts->fisbone.message_header_fields = malloc(len+1)) == NULL) {
+      return -1;
+    }
     snprintf (ts->fisbone.message_header_fields, len+1, CONTENT_TYPE_FMT, name);
     ts->fisbone.current_header_size = len+1;
   }
@@ -443,14 +497,17 @@ write_accum (OCState * state)
 
   /* Prime candidates */
   candidates = oggz_table_new ();
+
+  /* XXX: we offset the candidate indices by some small positive number to
+   * avoid storing 0, as OggzTable treats insertion of NULL as a deletion */
+#define CN_OFFSET (0x7)
+
   ntracks = oggz_table_size (state->tracks);
   for (i=0; i < ntracks; i++) {
     ts = oggz_table_nth (state->tracks, i, &serialno);
     if (ts != NULL && ts->page_accum != NULL) {
       ncandidates++;
-      /* XXX: we offset the candidate index by 7 to avoid storing 0, as
-       * OggzTable treats insertion of NULL as a deletion */
-      oggz_table_insert (candidates, serialno, (void *)0x7);
+      oggz_table_insert (candidates, serialno, (void *)CN_OFFSET);
       remaining += oggz_table_size (ts->page_accum);
     }
   }
@@ -463,7 +520,7 @@ write_accum (OCState * state)
     min_og = NULL;
     min_serialno = -1;
     for (i=0; i < ncandidates; i++) {
-      cn = ((int) oggz_table_nth (candidates, i, &serialno)) - 7;
+      cn = ((int) oggz_table_nth (candidates, i, &serialno)) - CN_OFFSET;
       ts = oggz_table_lookup (state->tracks, serialno);
       if (ts && ts->page_accum) {
         if (cn < oggz_table_size (ts->page_accum)) {
@@ -482,12 +539,16 @@ write_accum (OCState * state)
 
     if (min_og) {
       /* Increment index for minimum page */
-      oggz_table_insert (candidates, min_serialno, (void *)(min_cn+1+7));
+      oggz_table_insert (candidates, min_serialno,
+                         (void *)(min_cn+1+CN_OFFSET));
 
       /* Write out minimum page */
-      fwrite_ogg_page (state->outfile, min_og);
+      fwrite_ogg_page (state, min_og);
     }
 
+    /* Let's lexically forget about this CN_OFFSET silliness */
+#undef CN_OFFSET
+
     remaining--;
   }
 
@@ -508,6 +569,9 @@ write_accum (OCState * state)
 static int
 read_plain (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data);
 
+/* Write out the fisbones and accumulated pages before the chop point.
+ * This is called once by read_plain() below as soon as a page beyond the
+ * chop start is read. */
 static int
 chop_glue (OCState * state, OGGZ * oggz)
 {
@@ -559,8 +623,8 @@ read_plain (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
 #endif
 
   if (page_time < state->start) {
-    if ((gp = ogg_page_granulepos (og)) != -1)
-      ts->fisbone.start_granule = ogg_page_granulepos (og);
+    if ((gp = ogg_page_granulepos (OGG_PAGE_CONST(og))) != -1)
+      ts->fisbone.start_granule = ogg_page_granulepos (OGG_PAGE_CONST(og));
   } else if (page_time >= state->start &&
       (state->end == -1 || page_time <= state->end)) {
 
@@ -568,11 +632,11 @@ read_plain (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
       chop_glue (state, oggz);
     }
 
-    fwrite_ogg_page (state->outfile, og);
+    fwrite_ogg_page (state, og);
   } else if (state->end != -1.0 && page_time > state->end) {
     /* This is the first page past the end time; set EOS */
-    _ogg_page_set_eos ((ogg_page *)og);
-    fwrite_ogg_page (state->outfile, og);
+    _ogg_page_set_eos (og);
+    fwrite_ogg_page (state, og);
 
     /* Stop handling this track */
     oggz_set_read_page (oggz, serialno, NULL, NULL);
@@ -595,7 +659,6 @@ read_gs (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
   double page_time;
   ogg_int64_t granulepos, keyframe;
   int granuleshift, i, accum_size;
-  ogg_page * accum_og;
 
   page_time = oggz_tell_units (oggz) / 1000.0;
 
@@ -617,7 +680,7 @@ read_gs (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
     keyframe = granulepos >> granuleshift;
 
     if (keyframe != ts->prev_keyframe) {
-      if (ogg_page_continued(og)) {
+      if (ogg_page_continued(OGG_PAGE_CONST(og))) {
         /* If this new-keyframe page is continued, advance the page accumulator,
          * ie. recover earlier pages from this new GOP */
         accum_size = track_state_advance_page_accum (ts);
@@ -639,6 +702,56 @@ read_gs (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
   return OGGZ_CONTINUE;
 }
 
+static int
+read_dirac (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
+{
+  OCState * state = (OCState *)user_data;
+  OCTrackState * ts;
+  OCPageAccum * pa;
+  double page_time;
+  ogg_int64_t granulepos, keyframe, dist;
+  int granuleshift, i, accum_size;
+
+  page_time = oggz_tell_units (oggz) / 1000.0;
+
+  ts = oggz_table_lookup (state->tracks, serialno);
+  accum_size = oggz_table_size (ts->page_accum);
+
+  if (page_time >= state->start) {
+    /* Glue in fisbones, write out accumulated pages */
+    chop_glue (state, oggz);
+
+    /* Switch to the plain page reader */
+    oggz_set_read_page (oggz, serialno, read_plain, state);
+    return read_plain (oggz, og, serialno, user_data);
+  } /* else { ... */
+
+  granulepos = ogg_page_granulepos (OGG_PAGE_CONST(og));
+  if (granulepos != -1) {
+    granuleshift = oggz_get_granuleshift (oggz, serialno);
+    keyframe = granulepos >> granuleshift;
+    dist = ((keyframe & 0xff) << 8) | (granulepos & 0xff);
+
+    if (dist == 0) {
+      if (ogg_page_continued(OGG_PAGE_CONST(og))) {
+        /* If this new-keyframe page is continued, advance the page accumulator,
+         * ie. recover earlier pages from this new GOP */
+        accum_size = track_state_advance_page_accum (ts);
+      } else {
+        /* Otherwise, just clear the page accumulator */
+        track_state_remove_page_accum (ts);
+        accum_size = 0;
+      }
+    }
+  }
+
+  /* Add a copy of this to the page accumulator */
+  pa = page_accum_new (og, page_time);
+  oggz_table_insert (ts->page_accum, accum_size, pa);
+
+  return OGGZ_CONTINUE;
+}
+
 /*
  * OggzReadPageCallback read_headers
  *
@@ -653,22 +766,29 @@ read_headers (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
   fisbone_packet fisbone;
 
   content_type = oggz_stream_get_content(oggz, serialno);
-  if(content_type == OGGZ_CONTENT_SKELETON) {
+  switch (content_type) {
+  case OGGZ_CONTENT_SKELETON:
     if (fisbone_from_ogg_page (og, &fisbone) != -1) {
       if ((ts = oggz_table_lookup (state->tracks, fisbone.serial_no)) != NULL) {
         ts->fisbone.current_header_size = fisbone.current_header_size;
         ts->fisbone.message_header_fields = fisbone.message_header_fields;
       }
     }
-  } else {
-    fwrite_ogg_page (state->outfile, og);
-
+    break;
+  default:
     ts = oggz_table_lookup (state->tracks, serialno);
+    if (ts == NULL) break;
+
+    fwrite_ogg_page (state, og);
+
     ts->headers_remaining -= ogg_page_packets (OGG_PAGE_CONST(og));
 
     if (ts->headers_remaining <= 0) {
       if (state->start == 0.0 || oggz_get_granuleshift (oggz, serialno) == 0) {
         oggz_set_read_page (oggz, serialno, read_plain, state);
+      } else if (content_type == OGGZ_CONTENT_DIRAC) {
+        ts->page_accum = oggz_table_new();
+        oggz_set_read_page (oggz, serialno, read_dirac, state);
       } else {
         ts->page_accum = oggz_table_new();
         oggz_set_read_page (oggz, serialno, read_gs, state);
@@ -696,8 +816,14 @@ read_bos (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
         fishead_update (state, og);
       }
     } else {
-      ts = track_state_add (state->tracks, serialno);
-      fisbone_init (oggz, state, ts, serialno);
+      if ((ts = track_state_add (state->tracks, serialno)) == NULL) {
+        /* Out of memory */
+        return OGGZ_STOP_ERR;
+      }
+      if (fisbone_init (oggz, state, ts, serialno) == -1) {
+        /* Out of memory */
+        return OGGZ_STOP_ERR;
+      }
       ts->headers_remaining = ts->fisbone.nr_header_packet;
     }
 
@@ -722,6 +848,11 @@ chop (OCState * state)
 {
   OGGZ * oggz;
 
+  if (state == NULL || state->infilename == NULL) {
+    fprintf (stderr, "oggz-chop: Initialization state invalid\n");
+    return -1;
+  }
+
   state_init (state);
 
   if (strcmp (state->infilename, "-") == 0) {
@@ -730,14 +861,19 @@ chop (OCState * state)
     oggz = oggz_open (state->infilename, OGGZ_READ|OGGZ_AUTO);
   }
 
-  if (state->outfilename == NULL) {
-    state->outfile = stdout;
-  } else {
-    state->outfile = fopen (state->outfilename, "wb");
-    if (state->outfile == NULL) {
-      fprintf (stderr, "oggz-chop: unable to open output file %s\n",
-	       state->outfilename);
-      return -1;
+  if (oggz == NULL) return -1;
+
+  if (!state->dry_run) {
+    if (state->outfilename == NULL) {
+      state->outfile = stdout;
+    } else {
+      state->outfile = fopen (state->outfilename, "wb");
+      if (state->outfile == NULL) {
+        fprintf (stderr, "oggz-chop: unable to open output file %s\n",
+  	       state->outfilename);
+        oggz_close(oggz);
+        return -1;
+      }
     }
   }
 
@@ -756,12 +892,7 @@ chop (OCState * state)
 
   oggz_close (oggz);
 
-#if 0
-  if (state->do_skeleton)
-    oggz_close (state->skeleton_writer);
-#endif
-
-  if (state->outfilename != NULL) {
+  if (state->outfilename != NULL && !state->dry_run) {
     fclose (state->outfile);
   }
 
