@@ -35,7 +35,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <assert.h> /* , ja */
 #include <errno.h>
 #include <getopt.h>
 
@@ -72,6 +71,8 @@ usage (char * progname)
 {
   printf ("Usage: %s filename [options] tagname=tagvalue ...\n", progname);
   printf ("List or edit comments in an Ogg file.\n");
+  printf ("\nSee http://wiki.xiph.org/index.php/VorbisComment for usage recommendations.\n");
+  printf ("Note that VorbisComment metadata cannot be used with Dirac video tracks.\n");
   printf ("\nOutput options\n");
   printf ("  -l, --list             List the comments in the given file.\n");
   printf ("\nEditing options\n");
@@ -96,23 +97,41 @@ static OCData *
 ocdata_new ()
 {
   OCData *ocdata = malloc (sizeof (OCData));
-  assert (ocdata != NULL);
+
+  if (ocdata == NULL) return NULL;
+
   memset (ocdata, 0, sizeof (OCData));
 
   ocdata->do_all = 1;
 
   ocdata->storer = oggz_new (OGGZ_WRITE);
+  if (ocdata->storer == NULL)
+    goto err_storer;
   
   ocdata->seen_tracks = oggz_table_new ();
-  assert (ocdata->seen_tracks != NULL);
+  if (ocdata->seen_tracks == NULL)
+    goto err_seen_tracks;
 
   ocdata->serialno_table = oggz_table_new();
-  assert (ocdata->serialno_table != NULL);
+  if (ocdata->serialno_table == NULL)
+    goto err_serialno_table;
 
   ocdata->content_types_table = oggz_table_new();
-  assert (ocdata->content_types_table != NULL);
+  if (ocdata->content_types_table == NULL)
+    goto err_content_types_table;
   
   return ocdata;
+
+err_content_types_table:
+  free (ocdata->serialno_table);
+err_serialno_table:
+  free (ocdata->seen_tracks);
+err_seen_tracks:
+  free (ocdata->storer);
+err_storer:
+  free (ocdata);
+
+  return NULL;
 }
 
 static void 
@@ -132,6 +151,16 @@ ocdata_delete (OCData *ocdata)
   free (ocdata);
 }
 
+static void
+fail_dirac (OCData *ocdata)
+{
+  fprintf (stderr, "oggz-comment: Error: Comments cannot be stored in Dirac\n");
+
+  ocdata_delete (ocdata);
+
+  exit (1);
+}
+
 static int
 filter_stream_p (const OCData *ocdata, long serialno)
 {
@@ -162,14 +191,30 @@ read_bos (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
     return OGGZ_CONTINUE;
   }
 
-  /* Record this track in the seen_tracks table, unless it is skeleton.
+  /* Record this track in the seen_tracks table, unless it is Skeleton or Dirac.
    * We don't need to store any information about the track, just the fact
    * that it exists.
    * Store a dummy value (as NULL is not allowed in an OggzTable).
    */
   content = oggz_stream_get_content (oggz, serialno);
-  if (content != OGGZ_CONTENT_SKELETON)
+  switch (content) {
+  case OGGZ_CONTENT_SKELETON:
+    /* No VorbisComment for Skeleton, and no need to warn */
+    break;
+  case OGGZ_CONTENT_DIRAC:
+    /* No VorbisComment for Dirac, see:
+     * http://lists.xiph.org/pipermail/ogg-dev/2008-November/001277.html
+     */
+    if (ocdata->do_all) {
+      fprintf (stderr, "oggz-comment: Warning: Ignoring Dirac track, serialno %010lu\n",
+               serialno);
+    } else {
+      fail_dirac (ocdata);
+    }
+    break;
+  default:
     oggz_table_insert (ocdata->seen_tracks, serialno, (void *)0x01);
+  }
 
   ident = ot_page_identify (oggz, og, NULL);
   if (ident != NULL) {
@@ -210,9 +255,11 @@ static int
 read_page (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
 {
   OCData * ocdata = (OCData *)user_data;
+  size_t n;
 
-  fwrite (og->header, 1, og->header_len, ocdata->outfile);
-  fwrite (og->body, 1, og->body_len, ocdata->outfile);
+  n = fwrite (og->header, 1, og->header_len, ocdata->outfile);
+  if (n == (size_t)og->header_len)
+    n = fwrite (og->body, 1, og->body_len, ocdata->outfile);
 
   return OGGZ_CONTINUE;
 }
@@ -284,7 +331,7 @@ edit_comments (OCData * ocdata, char * outfilename)
 
   /* Set up writer, filling in ocdata for callbacks */
   if ((ocdata->writer = oggz_new (OGGZ_WRITE)) == NULL) {
-    printf ("Unable to create new writer\n");
+    fprintf (stderr, "Unable to create new writer: out of memory\n");
   }
 
   /* Set a page reader to process bos pages */
@@ -293,8 +340,10 @@ edit_comments (OCData * ocdata, char * outfilename)
   /* First, process headers packet-by-packet. */
   oggz_set_read_callback (ocdata->reader, -1, read_packet, ocdata);
   while ((n = oggz_read (ocdata->reader, 1024)) > 0) {
-    while (oggz_write_output (ocdata->writer, buf, n) > 0) {
-      fwrite (buf, 1, n, ocdata->outfile);
+    long nn;
+    while ((nn=oggz_write_output (ocdata->writer, buf, n)) > 0) {
+      if (fwrite (buf, 1, nn, ocdata->outfile) < (size_t)nn)
+        break;
     }
   }
 
@@ -325,9 +374,9 @@ read_comments(OGGZ *oggz, ogg_packet *op, long serialno, void *user_data)
   if (filter_stream_p (ocdata, serialno) && op->packetno == 1) {
     codec_name = oggz_stream_get_content_type(oggz, serialno);
     if (codec_name) {
-      printf ("%s: serialno %010ld\n", codec_name, serialno);
+      printf ("%s: serialno %010lu\n", codec_name, serialno);
     } else {
-      printf ("???: serialno %010ld\n", serialno);
+      printf ("???: serialno %010lu\n", serialno);
     }
 
     printf("\tVendor: %s\n", oggz_comment_get_vendor(oggz, serialno));
@@ -358,13 +407,17 @@ list_comments (OCData * ocdata)
 static void
 store_comment (OCData * ocdata, char * s)
 {
-  char * c, * name, * value;
+  char * c, * name, * value = NULL;
 
-  c = strchr (s, '=');
-  *c = '\0';
+  if (s == NULL) return;
 
   name = s;
-  value = c+1;
+
+  c = strchr (s, '=');
+  if (c != NULL) {
+    *c = '\0';
+    value = c+1;
+  }
 
   oggz_comment_add_byname (ocdata->storer, S_SERIALNO, name, value);
 }
@@ -386,33 +439,48 @@ main (int argc, char ** argv)
   long n;
   int i = 1;
 
+  char * optstring = "lo:dac:s:hv";
+
+#ifdef HAVE_GETOPT_LONG
+  static struct option long_options[] = {
+    {"list",     no_argument, 0, 'l'},
+    {"output",   required_argument, 0, 'o'},
+    {"delete",   no_argument, 0, 'd'},
+    {"all",      no_argument, 0, 'a'},
+    {"content-type", required_argument, 0, 'c'},
+    {"serialno", required_argument, 0, 's'},
+    {"help",     no_argument, 0, 'h'},
+    {"version",  no_argument, 0, 'v'},
+    {0,0,0,0}
+  };
+#endif
+
   ot_init ();
 
   progname = argv[0];
 
+  if (argc == 2 && !strncmp (argv[1], "-?", 2)) {
+#ifdef HAVE_GETOPT_LONG
+    ot_print_options (long_options, optstring);
+#else
+    ot_print_short_options (optstring);
+#endif
+    exit (0);
+  }
+
   if (argc < 3) {
     usage (progname);
     return (1);
   }
 
   ocdata = ocdata_new ();
+  if (ocdata == NULL) {
+    fprintf (stderr, "oggz-comment: out of memory\n");
+    exit (1);
+  }
 
   while (1) {
-    char * optstring = "lo:dac:s:hv";
-
 #ifdef HAVE_GETOPT_LONG
-    static struct option long_options[] = {
-      {"list",     no_argument, 0, 'l'},
-      {"output",   required_argument, 0, 'o'},
-      {"delete",   no_argument, 0, 'd'},
-      {"all",      no_argument, 0, 'a'},
-      {"content-type", required_argument, 0, 'c'},
-      {"serialno", required_argument, 0, 's'},
-      {"help",     no_argument, 0, 'h'},
-      {"version",  no_argument, 0, 'v'},
-      {0,0,0,0}
-    };
-
     i = getopt_long(argc, argv, optstring, long_options, NULL);
 #else
     i = getopt (argc, argv, optstring);
@@ -440,6 +508,9 @@ main (int argc, char ** argv)
       oggz_table_insert (ocdata->serialno_table, serialno, (void *)0x7);
       break;
     case 'c': /* content-type */
+      if (strcasecmp (optarg, "dirac") == 0) {
+        fail_dirac (ocdata);
+      }
       filter_content_types = 1;
       ocdata->do_all = 0;
       n = oggz_table_size (ocdata->content_types_table);
