@@ -35,13 +35,18 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <getopt.h>
+
 #include <oggz/oggz.h>
 
-#ifdef WIN32                                                                   
-#include <fcntl.h>    
+#include "dirac.h"
+#include "oggz_tools_dirac.h"
+
+#if defined (WIN32) || defined (__EMX__)
+#include <fcntl.h>
 #include <io.h>
 #define snprintf _snprintf
-#endif  
+#endif
 
 #ifdef HAVE_INTTYPES_H
 #  include <inttypes.h>
@@ -49,6 +54,7 @@
 #  define PRId64 "I64d"
 #endif
 
+
 static  ogg_uint32_t
 _le_32 (ogg_uint32_t i)
 {
@@ -131,8 +137,8 @@ ot_theora_info (unsigned char * data, long len)
 
   buf = malloc (80);
 
-  width = INT16_BE_AT(&data[10]) << 4;
-  height = INT16_BE_AT(&data[12]) << 4;
+  width = INT16_BE_AT(&data[15]);
+  height = INT16_BE_AT(&data[18]);
 
   snprintf (buf, 80,
 	    "\tVideo-Framerate: %.3f fps\n"
@@ -207,7 +213,7 @@ ot_flac_info (unsigned char * data, long len)
   version_major = data[5];
   version_minor = data[6];
 
-  samplerate = (ogg_int64_t) (data[27] << 12) | (data[28] << 4) | 
+  samplerate = (ogg_int64_t) (data[27] << 12) | (data[28] << 4) |
                ((data[29] >> 4)&0xf);
   channels = 1 + ((data[29] >> 1)&0x7);
 
@@ -269,6 +275,32 @@ ot_kate_info (unsigned char * data, long len)
   return buf;
 }
 
+static char *
+ot_dirac_info (unsigned char * data, long len)
+{
+  char * buf;
+  dirac_info *info;
+
+  /* read in useful bits from sequence header */
+  if (len < 24) return NULL;
+
+  buf = malloc (80);
+  info = malloc(sizeof(dirac_info));
+
+  dirac_parse_info(info, data, len);
+
+  snprintf (buf, 80,
+	    "\tVideo-Framerate: %.3f fps\n"
+	    "\tVideo-Width: %d\n\tVideo-Height: %d\n",
+	    (double)info->fps_numerator/ (double)info->fps_denominator,
+	    info->width, info->height);
+
+  free(info);
+
+  return buf;
+}
+
+
 static char *
 ot_skeleton_info (unsigned char * data, long len)
 {
@@ -303,7 +335,7 @@ ot_skeleton_info (unsigned char * data, long len)
 }
 
 static const OTCodecInfoFunc codec_ident[] = {
-  ot_theora_info, 
+  ot_theora_info,
   ot_vorbis_info,
   ot_speex_info,
   ot_oggpcm2_info,
@@ -315,7 +347,8 @@ static const OTCodecInfoFunc codec_ident[] = {
   NULL,             /* ANXDATA */
   ot_celt_info,     /* CELT */
   ot_kate_info,     /* KATE */
-  NULL              /* UNKOWN */
+  ot_dirac_info,    /* BBCD */
+  NULL              /* UNKNOWN */
 };
 
 const char *
@@ -329,9 +362,9 @@ ot_page_identify (OGGZ *oggz, const ogg_page * og, char ** info)
    * identify stream content using oggz_stream_get_content, identify
    * stream content name using oggz_stream_get_content_type
    */
-  
+
   serial_no = ogg_page_serialno((ogg_page *)og);
-  
+
   content = oggz_stream_get_content(oggz, serial_no);
   if (content == OGGZ_ERR_BAD_SERIALNO) return NULL;
 
@@ -410,6 +443,16 @@ ot_fprint_time (FILE * stream, double seconds)
   return fprintf (stream, "%s%02d:%02d:%06.3f", sign, hrs, min, sec);
 }
 
+void
+ot_dirac_gpos_parse (ogg_int64_t iframe, ogg_int64_t pframe,
+                     struct ot_dirac_gpos * dg)
+{
+  dg->pt = (iframe + pframe) >> 9;
+  dg->dist = ((iframe & 0xff) << 8) | (pframe & 0xff);
+  dg->delay = pframe >> 9;
+  dg->dt = (ogg_int64_t)dg->pt - dg->delay;
+}
+
 int
 ot_fprint_granulepos (FILE * stream, OGGZ * oggz, long serialno,
                       ogg_int64_t granulepos)
@@ -423,8 +466,17 @@ ot_fprint_granulepos (FILE * stream, OGGZ * oggz, long serialno,
     iframe = granulepos >> granuleshift;
     pframe = granulepos - (iframe << granuleshift);
 
-    ret = fprintf (stream, "%" PRId64 "|%" PRId64, iframe, pframe);
-  }
+    if (oggz_stream_get_content (oggz, serialno) != OGGZ_CONTENT_DIRAC) {
+      ret = fprintf (stream, "%" PRId64 "|%" PRId64, iframe, pframe);
+    } else {
+      struct ot_dirac_gpos dg;
+      ot_dirac_gpos_parse (iframe, pframe, &dg);
+      ret = fprintf (stream,
+		     "(pt:%u,dt:%" PRId64 ",dist:%hu,delay:%hu)",
+		     dg.pt, dg.dt, dg.dist, dg.delay);
+    }
+
+}
 
   return ret;
 }
@@ -433,9 +485,40 @@ void
 ot_init (void)
 {
 #ifdef _WIN32
-  /* We need to set stdin/stdout to binary mode */
+  /* We need to set stdin/stdout to binary mode on Win32 */
 
   _setmode( _fileno( stdin ), _O_BINARY );
   _setmode( _fileno( stdout ), _O_BINARY );
 #endif
+#ifdef __EMX__
+  /* We need to set stdin/stdout to binary mode on OS/2*/
+
+  setmode( fileno( stdin ), O_BINARY );
+  setmode( fileno( stdout ), O_BINARY );
+#endif
 }
+
+void
+ot_print_short_options (char * optstring)
+{
+  char *c;
+
+  for (c=optstring; *c; c++) {
+    if (*c != ':') printf ("-%c ", *c);
+  }
+
+  printf ("\n");
+}
+
+#ifdef HAVE_GETOPT_LONG
+void
+ot_print_options (struct option long_options[], char * optstring)
+{
+  int i;
+  for (i=0; long_options[i].name != NULL; i++)  {
+    printf ("--%s ", long_options[i].name);
+  }
+
+  ot_print_short_options (optstring);
+}
+#endif
